package sim.app.sensor_emitter;

import sim.util.MutableDouble2D;
import sim.util.Double2D;

/**
 * Points an emitter's signal at the given location.
 * Points straight up if the emitter and the target location are at the same position.
 *
 * @author Adam Campbell
 */
class FixedPointSignalController extends SignalController{

	/** location that this emitter will point to */
	private Double2D target;

	/** 
	 * Constructor that takes in a string array containing the point that the emitter's signal is directed at.
	 * The array should contain exactly three elements ('FixedPointSignalController [x] [y])
	 * No error checking is done here.
	 *
	 * @param signalControllerParams parameters given from the input file for this controller
	 */
	public FixedPointSignalController(final String[] signalControllerParams){
		this.target = new Double2D(Double.parseDouble(signalControllerParams[1]), Double.parseDouble(signalControllerParams[2]));
	}
	
	/**
	 * Points the emitter's signal towards its target.
	 *
	 * @param e emitter that is controller
	 * @param sim the simulation that the emitter is in
	 */
	protected void controlSignal(final Emitter e, final Simulation sim){
		
		// if the emitter is on top of the target location, then point the signal straight up (any arbitrary direction would do)
		if(this.target.equals(e.getPosition())){
			e.setEmitDirection(new MutableDouble2D(0, -1));
			return;
		}
		
		// point the emitter's signal towards its target
		// negate the y-value because java apps have the positive y-axis facing down
		final MutableDouble2D h = new MutableDouble2D(this.target.x - e.getPosition().x, -(this.target.y - e.getPosition().y));
		e.setEmitDirection(h.normalize());
	}
}