package sim.app.sensor_emitter;

import ec.util.MersenneTwisterFast;

/**
 * Used to assign the frequencies of a group of agents.
 * 
 * @author Adam Campbell
 */
class InitialFrequencyAssigner{

	/**
	 * Positions the agents according to the given params.
	 * No error checking is done in this function.
	 *
	 * @param agents list of agents
	 * @param assignerParams the positioner and additional parameters given for the 'SENSOR_INITIAL_FREQUENCY_ASSIGNER_n' or 'EMITTER_INITIAL_FREQUENCY_ASSIGNER_n' parameter
	 * @param parameterName used if warning message is printed out
	 * @param random the simulation's random number generator
	 */
	protected static void assignFrequencies(final Agent[] agents, final String[] assignerParams, final String parameterName, final MersenneTwisterFast random){

		if(assignerParams[0].equalsIgnoreCase("RandomAssigner")){
			InitialFrequencyAssigner.randomAssigner(agents, random);
		}else if(assignerParams[0].equalsIgnoreCase("HomogeneousAssigner")){
			InitialFrequencyAssigner.homogeneousAssigner(agents, assignerParams);
		}else{
			System.err.println("(InitialFrequencyAssigner.assignFrequencies) WARNING: Invalid parameters for " + parameterName);
		}
	}	

	/**
	 * Assigns all of the agents a random frequency.
	 *
	 * @param agents should contain rows*columns number of agents
	 * @param random the simulation's random number generator
	 */
	private static void randomAssigner(final Agent[] agents, final MersenneTwisterFast random){
		for(Agent a: agents){
			a.setFrequency(random.nextInt(Agent.getNumFrequencies()));
		}
	}

	/**
	 * Assign all agents to the same frequency.
	 * No error checking is done in this function.
	 * 
	 * <code>params</code> should contain exactly two values<br><br>
	 *   (<code>HomogeneousAssigner [frequency]</code>),<br><br>
	 *   where <code>frequency</code> is the frequency assigned to all the agents.
	 *
	 * @param agents list of agents
	 * @param params should contain exactly three values (String int int)
	 */
	private static void homogeneousAssigner(final Agent[] agents, final String[] params){
		final int frequency = Integer.parseInt(params[1]);
		for(Agent a: agents){
			a.setFrequency(frequency);
		}
	}
}