package sim.app.sensor_emitter;

import java.util.ArrayList;

/**
 * Implements Method1 from AAMAS 2009 paper (Campbell and Wu).
 * Using the incoming messages from other senors, this sensor counts how many sensors (including itself) are monitoring each frequency.
 * One of the frequencies that is the least represented is then randomly selected as the new frequency for this agent. 
 * 
 * @author Adam Campbell
 */
class Method1FrequencyUpdater extends FrequencyUpdater{

	/**
	 * Constructor
	 */
	public Method1FrequencyUpdater(){
	}
	
	/**
	 * Updates the sensor's frequency based on the Method1 algorithm from the AAMAS 2009 paper (Campbell and Wu).
	 *
	 * @param s the sensor
	 * @param sim the simulation that the sensor is in
	 */
	protected void update(final Sensor s, final Simulation sim){
	
		final ArrayList<Integer> lowestCounts = new ArrayList<Integer>();
		final int[] counts = s.frequencyCount(sim.sensorNetwork, true);
		int min = Integer.MAX_VALUE;
		
		// determine which frequency is the least represented
		for(int n: counts){
			min = Math.min(n, min);
		}
		
		// loop through frequencies to see which one(s) is the least represented
		for(int i = 0; i < counts.length; i++){
			if(counts[i] == min){
				lowestCounts.add(i); // frequency i is least represented
			}
		}
		
		// randomly choose from amongst the least represented frequencies
		s.setFrequency(lowestCounts.get(sim.random.nextInt(lowestCounts.size())));
	}
}