package sim.util_amc;

import sim.util.MutableDouble2D;

/**
 * MathFunctions contains various functions for arrays of numbers
 *
 * @author Adam Campbell
 */
public class MathFunctions{

	/**
	 * returns the angle in degrees between two normalized vectors.
	 *
	 * @param v1 first vector
	 * @param v2 second vector
	 * @return angle between v1 and v2
	 */
	public static double angleBetween(final MutableDouble2D v1, final MutableDouble2D v2){
		return v1.equals(v2) ? 0.0 : Math.toDegrees(java.lang.Math.acos(v1.dot(v2)));
	}
	
	/**
	 * computes the sum of values in an array of integers
	 *
	 * @param nums numbers to sum
	 * @return sum of numbers in given array
	 */
	public static int sum(final int[] nums){
		int sum = 0;
		for(int i = 0; i < nums.length; i++){
			sum += nums[i];
		}
		return sum;
	}

	/**
	 * computes the sum of values in an array of doubles
	 *
	 * @param nums numbers to sum
	 * @return sum of numbers in given array
	 */
	public static double sum(final double[] nums){
		double sum = 0;
		for(int i = 0; i < nums.length; i++){
			sum += nums[i];
		}
		return sum;
	}

	/**
	 * computes the average of the values in an array of integers
	 *
	 * @param nums numbers to average
	 * @return average value of numbers in given array
	 */
	public static double average(final int[] nums){
		return (double)MathFunctions.sum(nums)/nums.length;
	}

	/**
	 * computes the average of the values in an array of doubles
	 *
	 * @param nums to average
	 * @return average value of numbers in given array
	 */
	public static double average(final double[] nums){
		return MathFunctions.sum(nums)/nums.length;
	}

	/**
	 * computes the standard deviation of the values in an array of integers
	 *
	 * @param nums numbers to compute the standard deviation of
	 * @return standard deviation of numbers in given array
	 */
	public static double stdev(final int[] nums){
		final double avg = MathFunctions.average(nums);
		double val = 0.0;

		for(int i = 0; i < nums.length; i++){
			val += (nums[i]-avg)*(nums[i]-avg);
		}

		return Math.sqrt(val/(nums.length-1));
	}

	/**
	 * computes the standard deviation of the values in an array of doubles
	 *
	 * @param nums numbers to compute the standard deviation of
	 * @return standard deviation of numbers in given array
	 */
	public static double stdev(final double[] nums){
		final double avg = MathFunctions.average(nums);
		double val = 0.0;

		for(int i = 0; i < nums.length; i++){
			val += (nums[i]-avg)*(nums[i]-avg);
		}

		return Math.sqrt(val/(nums.length-1));
	}
}