package sim.app.sensor_emitter;

import sim.util_amc.ConfigParams;
import sim.portrayal.continuous.ContinuousPortrayal2D;
import sim.portrayal.network.NetworkPortrayal2D;
import sim.portrayal.network.SpatialNetwork2D;
import sim.portrayal.network.SimpleEdgePortrayal2D;
import sim.engine.SimState;
import sim.display.Display2D;
import sim.display.GUIState;
import sim.display.Console;
import sim.display.Controller;
import javax.swing.JFrame;

/**
 * Contains all of the visualization code for the simulation
 * Most of the code is directly adopted from MASON tutorials.
 *
 * @author Adam Campbell
 */
public class GraphicalUserInterface extends GUIState{

	private int displayWidth, displayHeight;
	private ConfigParams params;

	private Display2D display;
	private JFrame displayFrame;

	private ContinuousPortrayal2D sensorPortrayal = new ContinuousPortrayal2D();
	private ContinuousPortrayal2D emitterPortrayal = new ContinuousPortrayal2D();
	private NetworkPortrayal2D sensorCommPortrayal = new NetworkPortrayal2D();

	public static void main(String[] args){

		final GraphicalUserInterface gui = new GraphicalUserInterface();
		final Console c = new Console(gui);

		c.setVisible(true);
	}

	public GraphicalUserInterface(SimState state){

		super(state);

		this.params = new ConfigParams("run.conf");
		this.displayWidth = this.params.paramInt("DISPLAY_WIDTH");
		this.displayHeight = this.params.paramInt("DISPLAY_HEIGHT");
	}

	public GraphicalUserInterface(){
		this(new Simulation( System.currentTimeMillis()));
	}

	public static String getName() { return "Sensors and Emitters"; }
	
	public static String getInfo() { return "<h2>Sensors and Emitters</h2><p>System written in the Summer of 2009 by Adam Campbell.  acampbel@cs.ucf.edu"; }

	public Object getSimulationInspectedObject() { return state; }

	public void start(){
		super.start();
		this.setupPortrayals();
	}


	public void load(SimState state){
		super.load(state);
		this.setupPortrayals();
	}

	public void setupPortrayals(){
		final Simulation sim = (Simulation)state;

		// set up the portrayal used to draw connections between communicating sensors
		this.sensorCommPortrayal.setField(new SpatialNetwork2D(sim.sensorCont2D, sim.sensorNetwork));
		this.sensorCommPortrayal.setPortrayalForAll(new SimpleEdgePortrayal2D());

		// set up the sensor and emitter portrayals
		this.sensorPortrayal.setField(sim.sensorCont2D);
		this.emitterPortrayal.setField(sim.emitterCont2D);

		// reschedule the displayer
		this.display.reset();

		// redraw the display
		this.display.repaint();
	}

	public void init(Controller c){

		super.init(c);

		// make the display frame
		this.display = new Display2D(this.displayWidth,this.displayHeight,this,1);
		this.displayFrame = this.display.createFrame();
		this.displayFrame.setTitle("Environment");
		c.registerFrame(this.displayFrame);   // register the frame so it appears in the "Display" list
		this.displayFrame.setVisible(true);

		// add all of the portrayals to the display
		this.display.attach(this.emitterPortrayal, "Emitters");
		this.display.attach(this.sensorCommPortrayal, "Sensor Links"); 
		this.display.attach(this.sensorPortrayal, "Sensors");

		this.display.setBackdrop(this.params.paramColor("BACKGROUND_COLOR"));
	}

	public void quit(){
		super.quit();
		if (this.displayFrame != null) this.displayFrame.dispose();
		this.displayFrame = null;
		this.display = null;
	}
}
