package sim.app.sensor_emitter;

import sim.util_amc.ConfigParams;
import java.awt.Color;

/**
 * An extension of <code>ConfigParams</code> that has some functions for reading in simulation-specific
 *   parameters for the Sensor-Emitter Simulation.
 *
 * @author Adam Campbell
 */
class SenseEmitConfigParams extends ConfigParams{

	/**
	 * Constructor takes in parameter file name.
	 *
	 * @param fileName name of parameter file that is to be read in.
	 */
	public SenseEmitConfigParams(final String fileName){
		super(fileName);
	}

	/**
	 * Returns a list of colors representing each frequency for the sensors.
	 *
	 * @return list of colors for sensors
	 */
	protected Color[] sensorColorParam(){
		return this.frequencyColorParam("SENSOR");
	}

	/**
	 * Returns a list of colors representing each frequency for the emitters.
	 *
	 * @return list of colors for emitters
	 */
	protected Color[] emitterColorParam(){
		return this.frequencyColorParam("EMITTER");
	}
	
	/**
	 * Returns a list of colors representing each frequency for the sensors or emitters.
	 *
	 * As of now, this function provides little to no error checking. 
	 *
	 * @param agentType should be one of "SENSOR" or "EMITTER"
	 */
	private Color[] frequencyColorParam(final String agentType){

		final int n = this.paramInt("NUM_FREQUENCIES");
		final Color[] colors = new Color[n];

		for(int i = 1; i <= n; i++){
			colors[i-1] = this.paramColor(agentType + "_COLOR_" + i);
		}

		return colors;
	}
}